mod assign_property;
mod attributes;
mod conditional_branches;
mod conditional_widget;
mod if_branch;
mod if_condition;
mod match_arm;
mod properties;
mod property;
mod property_name;
mod returned_widget;
mod signal_handler;
mod top_level_widget;
mod view_widgets;
mod widget;
mod widget_func;
mod widget_func_method;
